@@ -3,12 +3,12 @@
 import json
 from functools import partial
 
-from sql import Union, As, Column, Null, Literal, With, Select
+from sql import As, Column, Literal, Null, Union, With
 from sql.aggregate import Min
 
 from trytond.config import config
-from trytond.pool import Pool, PoolMeta
 from trytond.model import ModelSQL, ModelView, fields
+from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 
 dumps = partial(json.dumps, separators=(',', ':'), sort_keys=True)
@@ -223,12 +223,6 @@ class Party(metaclass=PoolMeta):
     __name__ = 'party.party'
 
     relations = fields.One2Many('party.relation.all', 'from_', 'Relations')
-    distance = fields.Function(fields.Integer('Distance'), 'get_distance')
-
-    @classmethod
-    def __setup__(cls):
-        super().__setup__()
-        cls._order.insert(0, ('distance', 'ASC NULLS LAST'))
 
     @classmethod
     def _distance_query(cls, usages=None, party=None, depth=None):
@@ -240,6 +234,9 @@ class Party(metaclass=PoolMeta):
         context = transaction.context
         database = transaction.database
 
+        query = super()._distance_query(
+            usages=usages, party=party, depth=depth)
+
         if usages is None:
             usages = context.get('relation_usages', [])
         if party is None:
@@ -248,7 +245,7 @@ class Party(metaclass=PoolMeta):
             depth = context.get('depth', default_depth)
 
         if not party:
-            return
+            return query
 
         all_relations = RelationAll.__table__()
 
@@ -288,44 +285,9 @@ class Party(metaclass=PoolMeta):
                 & (distance.distance < depth)))
         distance.query.all_ = True
 
-        return (distance
-            .select(
-                distance.to, Min(distance.distance).as_('distance'),
-                group_by=[distance.to], with_=[distance])
-            | Select([Literal(party).as_('to'), Literal(0).as_('distance')]))
-
-    @classmethod
-    def order_distance(cls, tables):
-        party, _ = tables[None]
-        key = 'distance'
-        if key not in tables:
-            query = cls._distance_query()
-            if not query:
-                return []
-            join = party.join(query, type_='LEFT',
-                condition=query.to == party.id)
-            tables[key] = {
-                None: (join.right, join.condition),
-                }
-        else:
-            query, _ = tables[key][None]
-        return [query.distance]
-
-    @classmethod
-    def get_distance(cls, parties, name):
-        distances = {p.id: None for p in parties}
-        query = cls._distance_query()
+        relation_distance = distance.select(
+            distance.to, Min(distance.distance).as_('distance'),
+            group_by=[distance.to], with_=[distance])
         if query:
-            cursor = Transaction().connection.cursor()
-            cursor.execute(*query)
-            distances.update(cursor)
-        return distances
-
-
-class ContactMechanism(metaclass=PoolMeta):
-    __name__ = 'party.contact_mechanism'
-
-    @classmethod
-    def __setup__(cls):
-        super().__setup__()
-        cls._order.insert(0, ('party.distance', 'ASC NULLS LAST'))
+            relation_distance |= query
+        return relation_distance
